from django.db import models
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _


# from treenode.models import TreeNodeModel


class LanguageTypes(models.TextChoices):
    EN = "en"
    HU = "hu"


''' 
    ExerciseDevice
    ------------ 
'''


class ExerciseDevice(models.Model):
    exercise_device_id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=100)
    image_url = models.ImageField(upload_to='images/', help_text='The image size is 1366x768')
    description = models.TextField(max_length=1000, blank=True, null=True, help_text="English description here")
    sort = models.IntegerField(blank=True, null=True)
    place = models.BooleanField()

    class Meta:
        db_table = 'exercise_device'
        verbose_name = _("Exercise Device")
        verbose_name_plural = _("Exercise Devices")
        ordering = ['name']

    def image_tag(self):
        return mark_safe('<img src="/media/%s" width="30%" height="30%" />' % self.image_url)

    image_tag.short_description = 'Image'

    def __str__(self):
        return self.name


class ExerciseDeviceAlternative(models.Model):
    exercise_device_alternative_id = models.AutoField(primary_key=True)
    exercise_device_parent = models.ForeignKey(ExerciseDevice, on_delete=models.CASCADE,
                                               related_name='exercise_device_parent')
    exercise_device_child = models.ForeignKey(ExerciseDevice, on_delete=models.CASCADE,
                                              related_name='exercise_device_child')

    class Meta:
        db_table = 'exercise_device_alternative'
        verbose_name = _("Device Alternative")
        verbose_name_plural = _("Device Alternatives")
        unique_together = ['exercise_device_parent', 'exercise_device_child']


class ExerciseDeviceTranslation(models.Model):
    translation_id = models.AutoField(primary_key=True)
    exercise_device = models.ForeignKey(ExerciseDevice, on_delete=models.CASCADE)
    name = models.CharField(max_length=100)
    language_code = models.CharField(max_length=2, choices=LanguageTypes.choices, default=LanguageTypes.HU)

    class Meta:
        db_table = 'exercise_device_translation'
        verbose_name = _("Translation")
        verbose_name_plural = _("Translations")

    def __str__(self):
        return self.name


''' 
    ------------
    ExerciseTree
    ------------ 
'''


class ExerciseTree(models.Model):
    tree_id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=100, help_text='The name should be in English here')
    description = models.TextField(max_length=1000, blank=True, null=True, help_text='The description should be in '
                                                                                     'English here')
    image_url = models.ImageField(upload_to='images/', help_text='The menu image size is 1366x768')
    active = models.BooleanField(default=0, blank=True, null=True)

    class Meta:
        db_table = 'exercise_tree'
        verbose_name = _("Exercise Tree")
        verbose_name_plural = _("Exercise Tree Items")
        ordering = ['name']

    def __str__(self):
        return self.name


class ExerciseTreeTranslation(models.Model):
    translation_id = models.AutoField(primary_key=True)
    tree = models.ForeignKey(ExerciseTree, on_delete=models.CASCADE)
    language_code = models.CharField(max_length=2, choices=LanguageTypes.choices, default=LanguageTypes.HU)
    name = models.CharField(max_length=100)
    description = models.TextField(max_length=1000, blank=True, null=True)

    class Meta:
        db_table = 'exercise_tree_translation'
        verbose_name = _("Translation")
        verbose_name_plural = _("Translations")

    def __str__(self):
        return self.name


class ExerciseTreeParents(models.Model):
    # treenode_display_field = 'name'
    exercise_tree_parents_id = models.AutoField(primary_key=True)
    exercise_tree_parent = models.ForeignKey(ExerciseTree, on_delete=models.CASCADE,
                                             related_name='exercise_tree_parent',
                                             help_text=_('Parent menu'), blank=True)
    exercise_tree_child = models.ForeignKey(ExerciseTree, on_delete=models.CASCADE)

    class Meta:
        db_table = 'exercise_tree_parents'
        verbose_name = _("Menu Tree Representation")
        verbose_name_plural = _("Menu Tree Representation")
        unique_together = [['exercise_tree_parent', 'exercise_tree_child']]


''' 
    ExerciseTYPE
    ------------ 
'''


class ExerciseType(models.Model):
    class UnitTypes(models.TextChoices):
        REPEAT = 'repeat'
        SECOND = 'second'
        MINUTE = 'minute'
        METER = 'meter'
        CM = 'centimeter'
        PERCENT = 'percent'
        CALORIES = 'calories'
        KG = 'kilogram'

    exercise_type_id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=100, help_text='The name should be in English here')
    description = models.TextField(max_length=1000, blank=True, null=True, help_text='The description should be in '
                                                                                     'English here')
    unit = models.CharField(choices=UnitTypes.choices, default=UnitTypes.REPEAT, max_length=50, blank=True, null=True)
    unit_quantity = models.BooleanField(default=0, blank=True, null=True)
    unit_quantity_unit = models.CharField(choices=UnitTypes.choices, default=UnitTypes.KG, max_length=50, blank=True,
                                          null=True)
    active = models.BooleanField(default=0, blank=True, null=True)
    base = models.BooleanField(default=0, blank=True, null=True)
    parents = models.ManyToManyField(ExerciseTree, through='ExerciseTypeParents', related_name='ExerciseTree')

    class Meta:
        db_table = 'exercise_type'
        verbose_name = _("Exercise")
        verbose_name_plural = _("Exercises")
        ordering = ['name']

    def __str__(self):
        return self.name


class ExerciseTypeImage(models.Model):
    class ImageTypes(models.TextChoices):
        IMAGE = 'image'
        VIDEO = 'video'
        MENU = 'menu'

    image_id = models.AutoField(primary_key=True)
    exercise_type = models.ForeignKey(ExerciseType, on_delete=models.CASCADE)
    name = models.CharField(max_length=50, blank=True, null=True)
    type = models.CharField(choices=ImageTypes.choices, default=ImageTypes.IMAGE, max_length=50, blank=True, null=True)
    url = models.ImageField(upload_to='images/', help_text='The menu image size is 1366x768')

    class Meta:
        db_table = 'exercise_type_image'
        verbose_name = _("Image")
        verbose_name_plural = _("Images")

    def __str__(self):
        return self.name


class ExerciseTypeTranslation(models.Model):
    translation_id = models.AutoField(primary_key=True)
    exercise_type = models.ForeignKey(ExerciseType, on_delete=models.CASCADE)
    language_code = models.CharField(max_length=2, choices=LanguageTypes.choices, default=LanguageTypes.HU)
    name = models.CharField(max_length=100)
    description = models.TextField(max_length=1000, blank=True, null=True)

    class Meta:
        db_table = 'exercise_type_translation'
        verbose_name = _("Translation")
        verbose_name_plural = _("Translations")

    def __str__(self):
        return self.name


class ExerciseTypeDevice(models.Model):
    exercise_type_device_id = models.AutoField(primary_key=True)
    exercise_type = models.ForeignKey(ExerciseType, on_delete=models.CASCADE)
    exercise_device = models.ForeignKey(ExerciseDevice, on_delete=models.CASCADE)

    class Meta:
        db_table = 'exercise_type_device'
        verbose_name = _("Exercise Device")
        verbose_name_plural = _("Exercise Devices")


class ExerciseTypeAlternative(models.Model):
    exercise_type_alternative_id = models.AutoField(primary_key=True)
    exercise_type_parent = models.ForeignKey(ExerciseType, on_delete=models.CASCADE,
                                             related_name='exercise_type_parent')
    exercise_type_child = models.ForeignKey(ExerciseType, on_delete=models.CASCADE,
                                            related_name='exercise_type_child')

    class Meta:
        db_table = 'exercise_type_alternative'
        verbose_name = _("Exercise Alternative")
        verbose_name_plural = _("Exercise Alternatives")
        unique_together = [['exercise_type_parent', 'exercise_type_child']]


class ExerciseTypeParents(models.Model):
    exercise_type_parents_id = models.AutoField(primary_key=True)
    exercise_type = models.ForeignKey(ExerciseType, on_delete=models.CASCADE)
    exercise_tree = models.ForeignKey(ExerciseTree, on_delete=models.CASCADE)

    class Meta:
        db_table = 'exercise_type_parents'
        verbose_name = _("Exercise Parent")
        verbose_name_plural = _("Exercise Parents")
        unique_together = [['exercise_type', 'exercise_tree']]


class TemplateTypes(models.TextChoices):
    MINI_SET = "mini_test_set"
    SPECIAL = "special"


class ExercisePlanTemplate(models.Model):
    exercise_plan_template_id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=100, help_text='The name should be in English here')
    description = models.TextField(max_length=1000, blank=True, null=True, help_text='The description should be in '
                                                                                     'English here')
    template_type = models.CharField(max_length=20, choices=TemplateTypes.choices, default=TemplateTypes.SPECIAL)

    class Meta:
        db_table = 'exercise_plan_template'
        verbose_name = _("Exercise Plan Template")
        verbose_name_plural = _("Exercise Plan Templates")

    def __str__(self):
        return self.name


class ExercisePlanTemplateTranslation(models.Model):
    translation_id = models.AutoField(primary_key=True)
    exercise_plan_template = models.ForeignKey(ExercisePlanTemplate, on_delete=models.CASCADE)
    language_code = models.CharField(max_length=2, choices=LanguageTypes.choices, default=LanguageTypes.HU)
    name = models.CharField(max_length=100)
    description = models.TextField(max_length=1000, blank=True, null=True)

    class Meta:
        db_table = 'exercise_plan_template_translation'
        verbose_name = _("Translation")
        verbose_name_plural = _("Translations")

    def __str__(self):
        return self.name


class ExercisePlanTemplateDetail(models.Model):
    exercise_plan_template_detail_id = models.AutoField(primary_key=True)
    exercise_plan_template = models.ForeignKey(ExercisePlanTemplate, on_delete=models.CASCADE)
    exercise_type = models.ForeignKey(ExerciseType, on_delete=models.CASCADE)
    serie = models.IntegerField()
    quantity = models.FloatField()
    quantity_unit_quantity = models.FloatField()
    resting_time = models.TimeField()
    sort = models.IntegerField()

    class Meta:
        db_table = 'exercise_plan_template_detail'
        verbose_name = _("Exercise Plan Template Detail")
        verbose_name_plural = _("Exercise Plan Template Details")


class ExercisePlan(models.Model):
    exercise_plan_id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=100, help_text='The name should be in English here')
    description = models.TextField(max_length=1000, blank=True, null=True, help_text='The description should be in '
                                                                                     'English here')

    class Meta:
        db_table = 'exercise_plan'
        verbose_name = _("Exercise Plan")
        verbose_name_plural = _("Exercise Plans")

    def __str__(self):
        return self.name


class ExercisePlanTranslation(models.Model):
    translation_id = models.AutoField(primary_key=True)
    exercise_plan = models.ForeignKey(ExercisePlan, on_delete=models.CASCADE)
    language_code = models.CharField(max_length=2, choices=LanguageTypes.choices, default=LanguageTypes.HU)
    name = models.CharField(max_length=100)
    description = models.TextField(max_length=1000, blank=True, null=True)

    class Meta:
        db_table = 'exercise_plan_translation'
        verbose_name = _("Translation")
        verbose_name_plural = _("Translations")

    def __str__(self):
        return self.name


class ExercisePlanDetail(models.Model):
    exercise_plan_detail_id = models.AutoField(primary_key=True)
    exercise_plan = models.ForeignKey(ExercisePlan, on_delete=models.CASCADE)
    exercise_type = models.ForeignKey(ExerciseType, on_delete=models.CASCADE)
    serie = models.IntegerField()
    repeats = models.IntegerField()
    weight_equation = models.CharField(max_length=100)

    class Meta:
        db_table = 'exercise_plan_detail'
        verbose_name = _("Exercise Plan Detail")
        verbose_name_plural = _("Exercise Plan Details")


class ProductTypes(models.TextChoices):
    SUBS = "subscription"
    IN_APP_CURR = "in-app-currency"


class Product(models.Model):
    product_id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=50)
    description = models.TextField(max_length=1000, blank=True, null=True)
    type = models.CharField(max_length=15, choices=ProductTypes.choices, default=ProductTypes.SUBS)
    valid_from = models.DateField(blank=True, null=True)
    valid_to = models.DateField(blank=True, null=True)
    product_id_ios = models.CharField(max_length=50, blank=True, null=True)
    product_id_android = models.CharField(max_length=50, blank=True, null=True)
    price_ios = models.DecimalField(max_length=12, decimal_places=2, max_digits=12, blank=True)
    price_android = models.DecimalField(max_length=12, decimal_places=2, max_digits=12, blank=True)

    class Meta:
        db_table = 'product'
        verbose_name = _("Product")
        verbose_name_plural = _("Products")

    def __str__(self):
        return self.name


class Purchase(models.Model):
    purchase_id = models.AutoField(primary_key=True)
    product = models.ForeignKey(Product, on_delete=models.CASCADE)
    customer_id = models.IntegerField()
    date_add = models.DateField(blank=True, null=True)
    purchase_sum = models.DecimalField(decimal_places=2, max_digits=7)
    currency = models.CharField(max_length=3)

    class Meta:
        db_table = 'purchase'
        verbose_name = _("Purchase")
        verbose_name_plural = _("Purchases")


class Property(models.Model):
    property_id = models.AutoField(primary_key=True)
    property_name = models.CharField(max_length=100, help_text='The name should be in English here')
    property_unit = models.CharField(max_length=10)

    class Meta:
        db_table = 'property'
        verbose_name = _("Customer Property")
        verbose_name_plural = _("Customer Properties")

    def __str__(self):
        return self.property_name


class PropertyTranslation(models.Model):
    translation_id = models.AutoField(primary_key=True)
    property = models.ForeignKey(Property, on_delete=models.CASCADE)
    language_code = models.CharField(max_length=2, choices=LanguageTypes.choices, default=LanguageTypes.HU)
    property_name = models.CharField(max_length=100)

    class Meta:
        db_table = 'property_translation'
        verbose_name = _("Translation")
        verbose_name_plural = _("Translations")

    def __str__(self):
        return self.property_name


class Evaluation(models.Model):
    evaluation_id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=100, help_text='The name should be in English here')
    exercise_type = models.ForeignKey(ExerciseType, on_delete=models.CASCADE, verbose_name=_("exercise_type"))
    unit = models.CharField(max_length=50, verbose_name=_("unit"))

    class Meta:
        db_table = 'evaluation'
        verbose_name = _("Evaluation")
        verbose_name_plural = _("Evaluations")

    def __str__(self):
        return self.name


class EvaluationAttribute(models.Model):
    class SexTypes(models.TextChoices):
        MAN = 'm'
        WOMAN = 'w'

    class EvaluationTypes(models.TextChoices):
        ELITE = "elite"
        EXCELLENT = "excellent"
        GOOD = "good"
        ABOVE_AVERAGE = "above_average"
        AVERAGE = "average"
        BELOW_AVERAGE = "below_average"
        FAIR = "fair"
        POOR = "poor"
        VERY_POOR = "very_poor"

    evaluation_attr_id = models.AutoField(primary_key=True)
    evaluation = models.ForeignKey(Evaluation, on_delete=models.CASCADE, verbose_name=_("evaluation_foreign"))
    name = models.CharField(max_length=100, help_text='The name should be in English here', verbose_name=_("name"))
    sex = models.CharField(max_length=1, choices=SexTypes.choices, default=SexTypes.MAN, verbose_name=_("sex"))
    age_min = models.IntegerField(verbose_name=_("age_min"))
    age_max = models.IntegerField(verbose_name=_("age_max"))
    value_min = models.FloatField(verbose_name=_("value_min"))
    value_max = models.FloatField(verbose_name=_("value_max"))
    evaluation_text = models.CharField(max_length=50, choices=EvaluationTypes.choices, default=EvaluationTypes.AVERAGE,
                                       verbose_name=_("evaluation_text"))
    suggestion = models.TextField(max_length=1000, blank=True, null=True, help_text="English suggestion here",
                                  verbose_name=_("suggestion"))

    class Meta:
        db_table = 'evaluation_attribute'
        verbose_name = _("Evaluation Table")
        verbose_name_plural = _("Evaluation Tables")

    def __str__(self):
        return self.name
